/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@pd.infn.it                            |
 | WWW:    http://www.pd.infn.it/~loreti/mlo.html       |
 *------------------------------------------------------*

 "Improved" quicksort, with a short insertion run if the
 recursion falls under M elements; procedure adapted from
 Sedgewick's "Algorithms in C" (3rd Ed) - Addison Wesley.

 $Id: iqs.c,v 1.1 2000/02/07 17:36:51 loreti Exp $

 *------------------------------------------------------*/

#include <stddef.h>

/**
 | The following entries specialize this procedure for the
 | sorting of an array of integers in ascending order
**/

typedef int Item;               /* Objects to be sorted */
#define less(i,j) (*(i) < *(j)) /* Comparison method    */

#define M 11                    /* Starting point for recursive quicksort */

static int  partition(Item *, int, int);
static void quicksort(Item *, int, int);
static void insertion(Item *, int, int);
static void swap(Item *, Item *);
static void compSwap(Item *, Item *);

void sort(
  Item         *array,
  const size_t  N
) {
  quicksort(array, 0, N-1);
  insertion(array, 0, N-1);
}

static void quicksort(
  Item *array,
  int   left,
  int   right
) {
  int i;

  if (right-left < M) return;

  swap(array+(left+right)/2, array+right-1);
  compSwap(array+left, array+right-1);
  compSwap(array+left, array+right);
  compSwap(array+right-1, array+right);
  i = partition(array, left+1, right-1);
  quicksort(array, left, i-1);
  quicksort(array, i+1, right);
}

static int partition(
  Item *array,
  int   left,
  int   right
) {
  int i = left-1;
  int j = right;
  Item *v = array+right;

  for (;;) {
    while (less(array+(++i), v)) { }
    while (less(v, array+(--j))) if (j == left) break;
    if (i >= j) break;
    swap(array+i, array+j);
  }
  swap(array+i, array+right);
  return i;
}

static void insertion(
  Item *array,
  int   left,
  int   right
) {
  int i;

  for (i = left+1;   i <= right;   i++) compSwap(array+left, array+i);
  for (i = left+2;   i <= right;   i++) {
    int j = i;
    Item *v = array+i;

    while (less(v, array+j-1)) {
      array[j] = array[j-1];
      j--;
    }
    array[j] = *v;
  }
}

static void swap(
  Item *first,
  Item *second
) {
  Item temp = *first;
  *first = *second;
  *second = temp;
}

static void compSwap(
  Item *first,
  Item *second
) {
  if (*second < *first) {
    Item temp = *first;
    *first = *second;
    *second = temp;
  }
}
