/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@pd.infn.it                            |
 | WWW:    http://www.pd.infn.it/~loreti/mlo.html       |
 *------------------------------------------------------*

 The quicksort procedure has been adapted from: Kernighan
 and Ritchie - "The C| programming language", 2nd Ed.
 (Addison-Wesley) - pag. 110.

 $Id: quicksort.c,v 1.1 2000/02/07 17:36:51 loreti Exp $

 *------------------------------------------------------*/

#include <string.h>

/**
 | The following entries specialize this procedure for the
 | sorting of an array of integers in ascending order
**/

typedef int Item;               /* Objects to be sorted */
#define less(i,j) (*(i) < *(j)) /* Comparison method    */

static void quickSort(Item *, int, int);
static void swap(Item *, Item *);

void sort(
  Item         *array,          /* Array to be sorted */
  const size_t  N               /* Number of elements */
) {
  quickSort(array, 0, N-1);
}

static void quickSort(
  Item *array,                  /* Array to be sorted,  */
  int   left,                   /*   from array[left]   */
  int   right                   /*     to array[right]. */
){
  if (left < right) {
    int i, last;

    swap(array + (left+right)/2, array + (last = left));

    for (i = left+1;   i <= right;   i++) {
      if ( less(array+i, array+left) ) {
        swap(array+(++last),  array+i);
      }
    }

    swap(array+left, array+last);

    quickSort(array, left,   last-1);
    quickSort(array, last+1, right);
  }
}

static void swap(
  Item *first,
  Item *second
) {
  Item temp = *first;
  *first = *second;
  *second = temp;
}
