/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@pd.infn.it                            |
 | WWW:    http://www.pd.infn.it/~loreti/mlo.html       |
 *------------------------------------------------------*

 Short test program for the sorting procedures; fills an
 integer array with random numbers, sorts the array, and
 tests the result.  The array size and the number of
 iterations are read from the program parameters.

 $Id: sortTest.c,v 1.1 2000/02/07 17:36:51 loreti Exp $

 *------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

void sort(int *, const size_t);

static void check(const int *, const size_t, const unsigned long,
                  const time_t);
static void print(const int *, const size_t);

int main(
  int   argc,
  char *argv[]
) {
  size_t         N, n;          /* Elements to be sorted and loop index */
  unsigned long  nTries, trie;  /* Number of runs and loop index */
  time_t         now;           /* Local time (used to seed srand) */
  int           *array;         /* Array to be sorted */

/**
 | Test and decode program parameters
**/

  if (argc < 3) {
    printf("Usage: %s N nTries\n", argv[0]);
    return EXIT_FAILURE;

  } else {
    char *pc;

    N = strtoul(argv[1], &pc, 0);
    if (*pc != '\0'   &&   !isspace((unsigned char) *pc)) {
      printf("Invalid character in N=\"%s\"\n", argv[1]);
      return EXIT_FAILURE;
    }

    nTries = strtoul(argv[2], &pc, 0);
    if (*pc != '\0'   &&   !isspace((unsigned char) *pc)) {
      printf("Invalid character in nTries=\"%s\"\n", argv[2]);
      return EXIT_FAILURE;
    }
  }

/**
 | Initialization
**/

  if ((array = malloc(N * sizeof(int))) == 0) {
    puts("malloc error\n");
    return EXIT_FAILURE;
  }
  srand(now = time(0));

/**
 | Loop over different runs:
 | - fills the array with random integer values;
 | - sorts the array;
 | - tests the array for ascending ordering.
**/

  for (trie = 0;   trie < nTries;   trie++) {
    for (n = 0;   n < N;   n++) {
      array[n] = rand();
    }

/*    print(array, N); */
    sort(array, N);
/*    print(array, N); */
    check(array, N, trie, now);
  }

  free(array);
  return EXIT_SUCCESS;
}

static void check(
  const int           *array,
  const size_t         N,
  const unsigned long  trie,
  const time_t         seed
) {
  size_t i;

  for (i = 1;   i < N;   i++) {
    if (array[i-1] > array[i]) {
      printf("*** FAILED *** array[%lu] > array[%lu]\n",
             (unsigned long) i-1, (unsigned long) i);
      printf("*** Run nr. %lu, random seed = %lu\n", trie, seed);
      print(array, N);
      break;
    }
    
  }
}

static void print(
  const int *array,
  const size_t N
) {
  size_t i;

  printf("---------------------------------");
  for (i = 0;   i < N;   i++) {
    if ((i % 5) == 0) puts("");
    printf("%15d", array[i]);
  }
  puts("");
}
